from Orange.classification import SVMLearner
from Orange.data import Domain, DiscreteVariable, ContinuousVariable, Table
from Orange.preprocess import Normalize
import orangecontrib.conformal as cp
import sklearn.datasets

import numpy as np
np.random.seed(12345)


digits = sklearn.datasets.load_digits()
domain = Domain([ContinuousVariable(f"A{i}") for i in range(64)],
                            DiscreteVariable("digit", map(str, range(10))))
tab = Table(domain, digits.data, digits.target)
normalizer = Normalize(zero_based = True, norm_type = Normalize.NormalizeBySpan)
tab = normalizer(tab)


import matplotlib.pyplot as plt
plt.figure()
for i, img in enumerate(digits.images[:10], start = 1):
    plt.subplot(1, 10, i)
    plt.xticks([], []), plt.yticks([], [])
    plt.imshow(img, cmap = "binary")
#plt.show()
plt.savefig("Figures/digits10.eps")


learner = SVMLearner(probability = True)
nc = cp.nonconformity.InverseProbability(learner)
ic = cp.classification.InductiveClassifier(nc)

train, test = next(cp.evaluation.RandomSampler(tab, 2, 1))
print(len(train), len(test))
r = cp.evaluation.run_train_test(ic, 0.03, train, test)
multi = []
for example, pred, ref in zip(test, r.preds, r.refs):
    if len(pred.classes()) >= 2:
        multi.append((example, pred.classes(), ref))
print(r.accuracy(), len(multi))


from math import ceil
w, h = 3, ceil(len(multi)/3)
plt.figure()
for i, (image, cls, ref) in enumerate(multi, start = 1):
    plt.subplot(h, w, i)
    plt.xticks([], []), plt.yticks([], [])
    plt.title('{' + ', '.join(cls) + '} | ' + str(ref))
    plt.imshow(image.x.reshape((8, 8)), cmap = "binary")
plt.tight_layout()
#plt.show()
plt.savefig("Figures/digits.eps")
