import numpy as np
np.random.seed(12345)

print('Classification\n')

import Orange
import orangecontrib.conformal as cp

iris = Orange.data.Table('iris')
train, test = next(cp.evaluation.RandomSampler(iris, 1, 1))
train, calibrate = next(cp.evaluation.RandomSampler(train, 1, 1))
test_instance = Orange.data.Instance(test[-1])
print(test_instance)

lr = Orange.classification.LogisticRegressionLearner()
ip = cp.nonconformity.InverseProbability(lr)
icp = cp.classification.InductiveClassifier(ip, train, calibrate)

print(test_instance.get_class())
print(icp(test_instance, 0.1))
print(icp(test_instance, 0.02))

multiple = []
for instance in test:
    prediction = icp(instance, 0.05)
    if len(prediction) > 1:
        multiple.append((instance, prediction))
print(len(multiple), len(test))
for instance, prediction in multiple:
    print(instance, '->', prediction)


res = cp.evaluation.run_train_test(icp, 0.1, train, test, calibrate)
print(res.accuracy())
print(res.singleton_criterion())
print(res.empty_criterion())
print(res.multiple_criterion())

import matplotlib.pyplot as plt
plt.figure(figsize = (5, 4))
plt.xlabel('Petal length')
plt.ylabel('Petal width')
x = (test.get_column_view('petal length')[0])
y = (test.get_column_view('petal width')[0])
c = [['r', 'g', 'b'][int(y)] for y in test.Y]
s = [len(p.classes()) for p in res.preds]
a = [r in p.classes() for p, r in zip(res.preds, res.refs)]
for xi, yi, ci, si, ai in zip(x,y,c,s,a):
    plt.scatter(xi, yi,
                edgecolors = ci, facecolors = (ci if si==1 else 'none'),
                marker = 'o' if ai or si==0 else 'x')
for l, lc in zip(test.domain.class_var.values, ['r', 'g', 'b']):
    plt.scatter([], [], edgecolors = lc, facecolors = lc, label = l)
plt.legend()
plt.tight_layout()
plt.savefig("Figures/iris.eps")

print('\nRegression\n')

print('\nscikit-learn\n')

from sklearn import datasets
from sklearn.metrics import mean_absolute_error

boston = datasets.load_boston()
mask = np.random.choice([False, True], len(boston.data))
X_train, y_train = boston.data[mask], boston.target[mask]
X_test, y_test = boston.data[~mask], boston.target[~mask]

from sklearn.neighbors import KNeighborsRegressor
neigh = KNeighborsRegressor(n_neighbors = 10)
neigh.fit(X_train, y_train)

pred = neigh.predict(X_test)
print(mean_absolute_error(y_test, pred))


print('\nconformal\n')

import Orange

data = Orange.data.Table.from_numpy(None, boston.data, boston.target)
train, test = data[mask], data[~mask]
test_instance = test[-1]
print(test_instance)
print(neigh.predict(test_instance.x.reshape((1, -1))))

from Orange.distance import Euclidean
import orangecontrib.conformal as cp

nc = cp.nonconformity.AbsErrorKNN(Euclidean(), 10)
ccp = cp.regression.CrossRegressor(nc, 5, train)
print(ccp(test_instance, 0.1))
print(ccp(test_instance, 0.05))

res = cp.evaluation.run(ccp, 0.1, cp.evaluation.RandomSampler(data, 3, 2), rep = 3)
print(res.accuracy())
print(res.median_range())
