import Orange
import orangecontrib.conformal as cp

data = Orange.data.Table("friedman.tab")
n = len(data)
print("{} Instances, {} Features".format(n, len(data.domain.attributes)))
""" 1200 Instances, 5 Features """

import numpy as np
np.random.seed(0)
split = np.random.choice([True, False], n, p = [0.75, 0.25])
train, test = data[split], data[~split]
print("Train size: {}, Test size: {}".format(len(train), len(test)))
""" Train size: 907, Test size: 293 """


# visualize
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
x, y, z = data.get_column_view(0)[0], data.get_column_view(1)[0], data.get_column_view(2)[0]
fig = plt.figure(figsize = (10, 5))
ax = fig.add_subplot(121, projection = None)
ax.scatter(x, y, alpha = 1.0, c = data.Y, s = 30, cmap = plt.get_cmap('Greys'))
ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
ax.set_xlabel(r'$x_1$')
ax.set_ylabel(r'$x_2$')
ax = fig.add_subplot(122, projection = '3d')
ax.scatter(x, y, z, alpha = 1.0, c = data.Y, s = 30, cmap = plt.get_cmap('Greys'))
ax.w_xaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
ax.w_yaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
ax.w_zaxis.set_pane_color((1.0, 1.0, 1.0, 0.0))
ax.set_xlabel(r'$x_1$')
ax.set_ylabel(r'$x_2$')
ax.set_zlabel(r'$x_3$')
plt.tight_layout()
plt.savefig("Figures/friedman.eps")


ncs = [
    cp.nonconformity.AbsError(Orange.regression.LinearRegressionLearner()),
    cp.nonconformity.AbsError(Orange.regression.KNNRegressionLearner(n_neighbors = 20)),
    cp.nonconformity.AbsErrorNormalized(Orange.regression.LinearRegressionLearner(), k = 20),
    cp.nonconformity.AvgErrorKNN(k = 20),
    cp.nonconformity.ErrorModelNC(Orange.regression.LinearRegressionLearner(),
                                  Orange.regression.KNNRegressionLearner(n_neighbors = 20)),
    cp.nonconformity.LOORegrNC(Orange.regression.LinearRegressionLearner(), k = 20)
]

for nc in ncs:
    ccp = cp.regression.InductiveRegressor(nc)
    r = cp.evaluation.run_train_test(ccp, 0.1, train[0::2], test, train[1::2])
    print(nc, r.accuracy(), r.median_range(), r.interdecile_range())
"""
AbsError (linear regression) 0.8600682593856656 8.429026386113833 3.552713678800501e-15
AbsError (knn regression) 0.8873720136518771 7.403325277000004 3.552713678800501e-15
AbsErrorNormalized (linear regression, k=20) 0.856655290102389 8.268001594273333 1.4417061551941135
AvgErrorKNN (k=20) 0.8771331058020477 7.957073464107147 1.7152836899982553
ErrorModelNC (linear regression, knn regression) 0.8532423208191127 7.77145259004002 4.38867107204001
LOORegrNC 0.8805460750853242 5.160614621822205 3.9479179837227876
"""
